from copy import deepcopy
from datetime import datetime

from mower.data import agent_list
from mower.solvers.infra.base_mixin import BaseMixin
from mower.solvers.infra.enter_room import EnterRoomSolver
from mower.solvers.infra.filter import RIIC_Filter
from mower.solvers.infra.riic_free_fill import RIIC_FreeFillSolver
from mower.solvers.infra.riic_tag_choose import RIICTagChoose
from mower.utils import config
from mower.utils.character_recognize import operator_room_select
from mower.utils.graph.utils import SceneGraphSolver
from mower.utils.image import cropimg
from mower.utils.log import logger
from mower.utils.scene import Scene


class RIIC_ChooseSolver(SceneGraphSolver, BaseMixin):
    solver_name = "基建选人"

    def run(self, room: str, agents: list, wait_time: float = 0) -> bool:
        """
        Args:
            room (str): 房间
            agents (list): 选人名单
            wait_time (float, optional): 选人完成到按确定的时间
        """
        self.agents = agents
        self.room = room
        self.wait_time = wait_time

        self.success = False
        self.agents_copy = deepcopy(agents)
        self.clean = False
        self.filter = False
        self.agent = None
        self.tmp_left = None

        self.start_time = datetime.now()
        self.tap_time = datetime.now()
        self.tmp_swipe = None

        self.choosed = []

        logger.debug(f"待选名单:{self.agents}")

        super().run()  # 不一定选完，但一定结束流程了

        return len(self.choosed) == self.agents

    @property
    def interval(self):
        return config.screenshot_avg / 1000  # 截图延迟

    def choose_agents(self):
        if self.agent is None:
            if len(self.agents_copy) > 0:
                """
                self.agent =[干员名,是否选过tag,是否左滑过]
                """
                self.agent = [self.agents_copy.pop(0), False, False]
                logger.info(f"开始选择干员：{self.agent[0]}")

            else:
                if self.wait_time > 0:
                    self.sleep(self.check_wait_time())
                self.success = True
                self.tap("confirm_blue")

                return

        agents = operator_room_select(config.recog.img)
        agents_dict = dict(agents)

        if len(self.agents) > 1:
            for i in agents_dict:
                if self.is_choosed(agents_dict[i]) and agents_dict[i][0][0] < 1600:
                    if i not in self.agents and i not in self.choosed:
                        logger.info(f"存在错选 {i}")
                        self.tap("choose_agent/clear", interval=self.interval)
                        self.agents_copy = deepcopy(self.agents)
                        self.choosed = []
                        self.agent = None
                        return

        if self.tmp_swipe:
            logger.debug(
                f"tmp_swipe:{self.tmp_swipe[-1][1][0][0]}, {agents[-1][1][0][0]}"
            )
            if self.tmp_swipe[-1][0] == agents[-1][0]:
                if abs(self.tmp_swipe[-1][1][0][0] - agents[-1][1][0][0]) > 30:
                    self.tmp_swipe = agents
                    return
            self.tmp_swipe = None
        agent_name = self.agent[0]

        if agent_name == "Free":
            self.choosed.append(RIIC_FreeFillSolver().run())
            self.agent = None
            return

        if agent_name in agents_dict.keys():
            if self.is_choosed(agents_dict[agent_name]):
                logger.info(f"{agent_name}选择完成")
                self.choosed.append(agent_name)
                self.agent = None
            else:
                if agents_dict[agent_name][0][0] > 1650:
                    self.swipe_noinertia((1000, 540), (-500, 0), interval=0.5)
                    return

                logger.debug(f"tap {agent_name} ")
                self.ctap(agents_dict[agent_name], 2, self.interval, agent_name)
            return

        logger.debug(f"选tag和左滑 {self.agent}")
        if self.agent[1] is False:
            tag = agent_list[agent_name]["profession"]
            if agent_name == "阿米娅":
                tag = "ALL"

            if RIICTagChoose().run(tag):
                self.agent[1] = True
            return
        if self.agent[2] is False:
            if self.tmp_left is None:
                self.tmp_left = list(agents_dict.keys())[0]
                self.swipe_noinertia((1000, 540), (300, 0), interval=0.5)
                return
            elif self.tmp_left != list(agents_dict.keys())[0]:
                self.swipe_noinertia((1000, 540), (1000, 0), interval=0.5)
                self.tmp_left = list(agents_dict.keys())[0]
                return
            self.agent[2] = True
            return

        # 右滑
        self.before_swipe = agents
        self.tmp_swipe = self.swipe_update(
            [(1000, 540), (-2500, 0)],
            [600],
            self.interval,
            interval=self.interval,
            func=operator_room_select,
        )
        logger.debug(f"右滑 {self.before_swipe},{self.tmp_swipe}")
        if self.tmp_swipe:
            if self.tmp_swipe[-1][0] == self.before_swipe[-1][0]:
                if self.tmp_swipe[-1][1][0][0] == self.before_swipe[-1][1][0][0]:
                    logger.error(RuntimeError("滑动失败"))
                    return True
                else:
                    self.tmp_swipe[-1][1][0][0] != self.before_swipe[-1][1][0][0]
                    raise RuntimeError(f"缺少干员 {self.agent[0]}")

    def is_choosed(self, scope) -> bool:
        if (
            cropimg(
                config.recog.img,
                [
                    [scope[0][0] + 0, scope[0][1] - 128],
                    [scope[0][0] + 1, scope[0][1] - 127],
                ],
            )[0][0][0]
            < 50
        ) and cropimg(
            config.recog.img,
            [
                [scope[0][0] + 0, scope[1][1] + 212],
                [scope[0][0] + 1, scope[1][1] + 213],
            ],
        )[0][0][0] < 50:
            return True
        return False

    def check_wait_time(self) -> float:
        return max(
            self.wait_time - (datetime.now() - self.start_time).total_seconds(), 0
        )

    def check_room(self):
        if (
            (scene := self.scene()) == Scene.INFRA_DETAILS
        ) and not EnterRoomSolver().detect_room() == self.room:
            logger.info(self.room)
            EnterRoomSolver().run(self.room)
        elif (
            scene in [Scene.ORDER_LIST, Scene.FACTORY_ROOMS]
            and not self.detect_room_inside() == self.room
        ):
            EnterRoomSolver().run(self.room)

    def transition(self) -> bool:
        if (datetime.now() - self.tap_time).total_seconds() > 300:
            EnterRoomSolver().run(self.room)
            return True
        if (scene := self.scene()) == Scene.RIIC_OPERATOR_SELECT:
            if pos := self.find("choose_agent/open_profession"):
                self.ctap(pos, interval=self.interval)
                return
            if self.filter is False:
                if RIIC_Filter().run("技能", ascending=False):
                    self.filter = True
            if self.clean is False and len(self.agents) > 1:
                agents_dict = dict(operator_room_select(config.recog.img))
                for i in agents_dict:
                    if self.is_choosed(agents_dict[i]):
                        logger.debug(f"开始选人前先清空干员 {i}")
                        self.tap("choose_agent/clear", interval=self.interval)
                        return
                self.clean = True

            return self.choose_agents()

        elif self.find("room_detail"):
            if self.success:
                return True
            else:
                self.ctap((1400, 200))
        elif scene in [Scene.ORDER_LIST, Scene.FACTORY_ROOMS]:
            if self.success:
                return True
            else:
                self.tap((500, 970))
        elif scene == Scene.INFRA_ARRANGE_ORDER:
            self.scene_graph_step(Scene.RIIC_OPERATOR_SELECT)
        elif scene == Scene.INFRA_ARRANGE_CONFIRM:
            self.scene_graph_step(Scene.RIIC_OPERATOR_SELECT)
        elif scene in self.waiting_scene:
            self.waiting_solver()
        else:
            EnterRoomSolver().run(self.room)
